/*
 * $Header: //depot/php_ext/cTemplate/php_cTemplate.cpp#6 $
 */

/* {{{ include files */
extern "C" {
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_cTemplate.h"
}

#include "cTemplate.h"
#include "cTemplateDict.h"
/* }}} */

/* {{{ static variable */
static int le_cTemplate;
static int le_cTemplateDict;
static zend_class_entry *ctemplate_class_entry_ptr;
static zend_class_entry *ctemplate_dict_class_entry_ptr;
/* }}} */

/* {{{ function_entry */
zend_function_entry cTemplate_functions[] = {
    PHP_FE (ctemplate_setrootpath, NULL)
    PHP_FE (ctemplate_reload, NULL)
    PHP_FE (ctemplate_clearcache, NULL)
    {NULL, NULL, NULL}
};

static zend_function_entry ctemplate_class_functions[] = {
    PHP_FE (ctemplate, NULL)
    PHP_FALIAS (expand, t_Expand, NULL)
    PHP_FALIAS (dump, t_Dump, NULL)
    PHP_FALIAS (state, t_state, NULL)
    PHP_FALIAS (template_file, t_template_file, NULL)
    PHP_FALIAS (reloadifchanged, t_ReloadIfChanged, NULL)
    PHP_FALIAS (template_root_directory, t_template_root_directory, NULL)
    PHP_FALIAS (writeheaderentries, t_WriteHeaderEntries, NULL)
    {NULL, NULL, NULL}
};

static zend_function_entry ctemplate_dict_class_functions[] = {
    PHP_FE (ctemplate_dict, NULL)
    PHP_FALIAS (setvalue, d_SetValue, NULL)
    PHP_FALIAS (setescapedvalue, d_SetEscapedValue, NULL)
    PHP_FALIAS (setglobalvalue, d_SetGlobalValue, NULL)
    PHP_FALIAS (addsectiondictionary, d_AddSectionDictionary, NULL)
    PHP_FALIAS (showsection, d_ShowSection, NULL)
    PHP_FALIAS (setvalueandshowsection, d_SetValueAndShowSection, NULL)
    PHP_FALIAS (setescapedvalueandshowsection, d_SetEscapedValueAndShowSection, NULL)
    PHP_FALIAS (addincludedictionary, d_AddIncludeDictionary, NULL)
    PHP_FALIAS (setfilename, d_SetFilename, NULL)
    PHP_FALIAS (dump, d_Dump, NULL)
    PHP_FALIAS (dumptostring, d_DumpToString, NULL)
    PHP_FALIAS (setannotateoutput, d_SetAnnotateOutput, NULL)
    {NULL, NULL, NULL}
};
/* }}} */

/* {{{ module entry */
zend_module_entry cTemplate_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
    STANDARD_MODULE_HEADER,
#endif
    "cTemplate",
    cTemplate_functions,
    PHP_MINIT(cTemplate),
    PHP_MSHUTDOWN(cTemplate),
    NULL,
    NULL,
    PHP_MINFO(cTemplate),
#if ZEND_MODULE_API_NO >= 20010901
    "$Revision: #6 $",
#endif
    STANDARD_MODULE_PROPERTIES
};
/* }}} */

/* {{{ resource allocate function */
static void * _return_resource (zval *wrapper, int rsrc_type TSRMLS_DC)
{
    void *obj = NULL;
    zval **handle;
    int type;

    if (Z_TYPE_P(wrapper) != IS_OBJECT)
    {
        php_error_docref (NULL TSRMLS_CC, E_ERROR, "Wrapper is not an object");
    }

    if (zend_hash_find (Z_OBJPROP_P(wrapper), "id", sizeof ("id"), (void **) &handle) == FAILURE)
    {
        php_error_docref (NULL TSRMLS_CC, E_ERROR, "Underlying object missing");
    }

    obj = zend_list_find (Z_LVAL_PP (handle), &type);
    if (!obj || (type != rsrc_type))
    {
        php_error_docref (NULL TSRMLS_CC, E_ERROR, "Underlying object missing");
    }
    return obj;
}

static void _free_ctemplate_resource (zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
    cTemplate *obj = static_cast <cTemplate *> (rsrc->ptr);

    if (obj)
    {
        delete obj;
        rsrc->ptr = NULL;
    }
}

static void _free_ctemplate_dict_resource (zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
    cTemplateDict *obj = static_cast <cTemplateDict *> (rsrc->ptr);

    if (obj)
    {
        delete obj;
        rsrc->ptr = NULL;
    }
}

static zval *_add_resource (zval *wrapper, void *obj, int rsrc_type TSRMLS_DC)
{
    zval *handle;

    if (rsrc_type == le_cTemplate)
        object_init_ex (wrapper, ctemplate_class_entry_ptr);
    else if (rsrc_type == le_cTemplateDict)
        object_init_ex (wrapper, ctemplate_dict_class_entry_ptr);

    MAKE_STD_ZVAL (handle);
    ZEND_REGISTER_RESOURCE (handle, obj, rsrc_type);
    zend_hash_update (Z_OBJPROP_P (wrapper), "id", sizeof ("id"), &handle, sizeof (zval *), NULL);
    return (wrapper);
}

/* }}} */

/* {{{ INIT */
extern "C" {
#ifdef COMPILE_DL_CTEMPLATE
ZEND_GET_MODULE(cTemplate)
#endif
}

PHP_MINIT_FUNCTION(cTemplate)
{
    zend_class_entry ctemplate_ce;
    zend_class_entry ctemplate_dict_ce;
    
    le_cTemplate = zend_register_list_destructors_ex (_free_ctemplate_resource, NULL, "cTemplate", module_number);
    le_cTemplateDict = zend_register_list_destructors_ex (_free_ctemplate_dict_resource, NULL, "cTemplate_Dict", module_number);

    INIT_CLASS_ENTRY (ctemplate_ce, "cTemplate", ctemplate_class_functions);
    INIT_CLASS_ENTRY (ctemplate_dict_ce, "cTemplate_Dict", ctemplate_dict_class_functions);

    ctemplate_class_entry_ptr = zend_register_internal_class (&ctemplate_ce TSRMLS_CC);
    ctemplate_dict_class_entry_ptr = zend_register_internal_class (&ctemplate_dict_ce TSRMLS_CC);

    REGISTER_LONG_CONSTANT ("DO_NOT_STRIP", DO_NOT_STRIP, CONST_CS | CONST_PERSISTENT);
    REGISTER_LONG_CONSTANT ("STRIP_BLANK_LINES", STRIP_BLANK_LINES, CONST_CS | CONST_PERSISTENT);
    REGISTER_LONG_CONSTANT ("STRIP_WHITESPACE", STRIP_WHITESPACE, CONST_CS | CONST_PERSISTENT);
    REGISTER_LONG_CONSTANT ("NUM_STRIPS", NUM_STRIPS, CONST_CS | CONST_PERSISTENT);
    REGISTER_LONG_CONSTANT ("TS_UNUSED", TS_UNUSED, CONST_CS | CONST_PERSISTENT);
    REGISTER_LONG_CONSTANT ("TS_EMPTY", TS_EMPTY, CONST_CS | CONST_PERSISTENT);
    REGISTER_LONG_CONSTANT ("TS_ERROR", TS_ERROR, CONST_CS | CONST_PERSISTENT);
    REGISTER_LONG_CONSTANT ("TS_READY", TS_READY, CONST_CS | CONST_PERSISTENT);
    REGISTER_LONG_CONSTANT ("TS_RELOAD", TS_RELOAD, CONST_CS | CONST_PERSISTENT);
    REGISTER_LONG_CONSTANT ("HTML_ESCAPE", 0, CONST_CS | CONST_PERSISTENT);
    REGISTER_LONG_CONSTANT ("XML_ESCAPE", 1, CONST_CS | CONST_PERSISTENT);
    REGISTER_LONG_CONSTANT ("JAVASCRIPT_ESCAPE", 2, CONST_CS | CONST_PERSISTENT);
    return SUCCESS;
}

PHP_MSHUTDOWN_FUNCTION(cTemplate)
{
    return SUCCESS;
}

PHP_MINFO_FUNCTION(cTemplate)
{
    php_info_print_table_start();
    php_info_print_table_header(2, "cTemplate support", "enabled");
    php_info_print_table_end();
}
/* }}} */

/* {{{ ctemplate */
PHP_FUNCTION(ctemplate)
{
    cTemplate * ctemplate;
    const char *fn;
    int fn_len;
    long flags;

    if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "sl", &fn, &fn_len, &flags) == FAILURE)
        RETURN_FALSE;

    ctemplate = new cTemplate;
    ctemplate->t = Template::GetTemplate (fn, (Strip)flags);

    if (ctemplate->t != NULL)
    {
        _add_resource (getThis(), ctemplate, le_cTemplate TSRMLS_CC);
        RETURN_TRUE;
    }
    RETURN_FALSE;
}
/* }}} */

/* {{{ ctemplate_setrootpath */
PHP_FUNCTION(ctemplate_setrootpath)
{
    const char *dir = NULL;
    int dir_len;
    zend_bool b;

    if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "s", &dir, &dir_len) == FAILURE)
        RETURN_FALSE;

    b = Template::SetTemplateRootDirectory (dir);
    RETURN_BOOL (b);
}
/* }}} */

/* {{{ ctemplate_reload */
PHP_FUNCTION(ctemplate_reload)
{
    if (ZEND_NUM_ARGS() != 0)
    {
        WRONG_PARAM_COUNT;
    }

    Template::ReloadAllIfChanged();
    RETURN_TRUE;
}
/* }}} */

/* {{{ ctemplate_clearcache */
PHP_FUNCTION(ctemplate_clearcache)
{
    if (ZEND_NUM_ARGS() != 0)
    {
        WRONG_PARAM_COUNT;
    }

    Template::ClearCache ();
    RETURN_TRUE;
}
/* }}} */

/* {{{ t_Expand */
PHP_FUNCTION(t_Expand)
{
    zval *d;
    cTemplate *ctemplate = NULL;
    cTemplateDict *cd = NULL;
    string ret;

    if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "o", &d) == FAILURE)
        RETURN_FALSE;

    ctemplate = (cTemplate *)_return_resource (getThis(), le_cTemplate TSRMLS_CC);
    if (ctemplate == NULL)
    {
        php_error (E_WARNING, "t_Expand: error getting object");
        RETURN_FALSE;
    }

    cd = (cTemplateDict *)_return_resource (d, le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "t_Expand: error getting dict object");
        RETURN_FALSE;
    }

    if (cd->root == true)
        ctemplate->t->Expand (&ret, &(cd->d));
    else
        ctemplate->t->Expand (&ret, cd->p);

    RETURN_STRINGL ((char *)ret.c_str(), ret.length(), 1);
}
/* }}} */

/* {{{ t_Dump */
PHP_FUNCTION(t_Dump)
{
    const char *fn = NULL;
    int fn_len;
    cTemplate *ctemplate = NULL;

    if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "s", &fn, &fn_len) == FAILURE)
        RETURN_FALSE;

    ctemplate = (cTemplate *)_return_resource (getThis(), le_cTemplate TSRMLS_CC);
    if (ctemplate == NULL)
    {
        php_error (E_WARNING, "t_Dump: error getting object");
        RETURN_FALSE;
    }

    ctemplate->t->Dump (fn);
    RETURN_TRUE;
}
/* }}} */

/* {{{ t_state */
PHP_FUNCTION(t_state)
{
    cTemplate *ctemplate = NULL;

    if (ZEND_NUM_ARGS() != 0)
    {
        WRONG_PARAM_COUNT;
    }

    ctemplate = (cTemplate *)_return_resource (getThis(), le_cTemplate TSRMLS_CC);
    if (ctemplate == NULL)
    {
        php_error (E_WARNING, "t_state: error getting object");
        RETURN_FALSE;
    }

    RETURN_LONG (ctemplate->t->state());
}
/* }}} */

/* {{{ t_template_file */
PHP_FUNCTION(t_template_file)
{
    cTemplate *ctemplate = NULL;

    if (ZEND_NUM_ARGS() != 0)
    {
        WRONG_PARAM_COUNT;
    }

    ctemplate = (cTemplate *)_return_resource (getThis(), le_cTemplate TSRMLS_CC);
    if (ctemplate == NULL)
    {
        php_error (E_WARNING, "t_template_file: error getting object");
        RETURN_FALSE;
    }

    RETURN_STRING ((char *)ctemplate->t->template_file(), 1);
}
/* }}} */

/* {{{ t_ReloadIfChanged */
PHP_FUNCTION(t_ReloadIfChanged)
{
    cTemplate *ctemplate = NULL;
    zend_bool b;

    if (ZEND_NUM_ARGS() != 0)
    {
        WRONG_PARAM_COUNT;
    }

    ctemplate = (cTemplate *)_return_resource (getThis(), le_cTemplate TSRMLS_CC);
    if (ctemplate == NULL)
    {
        php_error (E_WARNING, "t_ReloadIfChanged: error getting object");
        RETURN_FALSE;
    }

    b = ctemplate->t->ReloadIfChanged();
    RETURN_BOOL (b);
}
/* }}} */

/* {{{ t_template_root_directory */
PHP_FUNCTION(t_template_root_directory)
{
    string ret;

    if (ZEND_NUM_ARGS() != 0)
    {
        WRONG_PARAM_COUNT;
    }

    ret = Template::template_root_directory ();
    RETURN_STRINGL ((char *)ret.c_str(), ret.length(), 1);
}
/* }}} */

/* {{{ t_WriteHeaderEntries */
PHP_FUNCTION(t_WriteHeaderEntries)
{
    cTemplate *ctemplate = NULL;
    string ret;

    if (ZEND_NUM_ARGS() != 0)
    {
        WRONG_PARAM_COUNT;
    }

    ctemplate = (cTemplate *)_return_resource (getThis(), le_cTemplate TSRMLS_CC);
    if (ctemplate == NULL)
    {
        php_error (E_WARNING, "t_WriteHeaderEntries: error getting object");
        RETURN_FALSE;
    }

    ctemplate->t->WriteHeaderEntries (&ret);
    RETURN_STRINGL ((char *)ret.c_str(), ret.length(), 1);
}
/* }}} */

/* {{{ ctemplate_dict */
PHP_FUNCTION(ctemplate_dict)
{
    cTemplateDict * cd;

    if (ZEND_NUM_ARGS() != 0)
    {
        WRONG_PARAM_COUNT;
    }

    cd = new cTemplateDict;

    if (cd != NULL)
    {
        _add_resource (getThis(), cd, le_cTemplateDict TSRMLS_CC);
        RETURN_TRUE;
    }
    RETURN_FALSE;
}
/* }}} */

/* {{{ d_SetValue */
PHP_FUNCTION(d_SetValue)
{
    cTemplateDict *cd = NULL;
    const char *key = NULL, *val = NULL;
    int key_len, val_len;
    long lval;

    cd = (cTemplateDict *)_return_resource (getThis(), le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "d_SetValue: error getting object");
        RETURN_FALSE;
    }

    if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "sl", &key, &key_len, &lval) == SUCCESS)
    {
        if (cd->root == true)
            cd->d.SetIntValue (key, lval);
        else
            cd->p->SetIntValue (key, lval);
        RETURN_TRUE;
    }
    else if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "ss", &key, &key_len, &val, &val_len) == SUCCESS)
    {
        if (cd->root == true)
            cd->d.SetValue (key, val);
        else
            cd->p->SetValue (key, val);
        RETURN_TRUE;
    }
    else
        RETURN_FALSE;
}
/* }}} */

/* {{{ d_SetEscapedValue */
PHP_FUNCTION(d_SetEscapedValue)
{
    cTemplateDict *cd = NULL;
    const char *key = NULL, *val = NULL;
    int key_len, val_len;
    long e;

    cd = (cTemplateDict *)_return_resource (getThis(), le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "d_SetEscapedValue: error getting object");
        RETURN_FALSE;
    }

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ssl", &key, &key_len, &val, &val_len, &e) == FAILURE)
        RETURN_FALSE;

    switch (e)
    {
        case 1:
            if (cd->root == true)
                cd->d.SetEscapedValue (key, val, TemplateDictionary::xml_escape);
            else
                cd->p->SetEscapedValue (key, val, TemplateDictionary::xml_escape);
            break;
        case 2:
            if (cd->root == true)
                cd->d.SetEscapedValue (key, val, TemplateDictionary::javascript_escape);
            else
                cd->p->SetEscapedValue (key, val, TemplateDictionary::javascript_escape);
            break;
        case 0:
        default:
            if (cd->root == true)
                cd->d.SetEscapedValue (key, val, TemplateDictionary::html_escape);
            else
                cd->p->SetEscapedValue (key, val, TemplateDictionary::html_escape);
    }
    RETURN_TRUE;
}
/* }}} */

/* {{{ d_SetValueAndShowSection */
PHP_FUNCTION(d_SetValueAndShowSection)
{
    cTemplateDict *cd = NULL;
    const char *key = NULL, *val = NULL, sec = NULL;
    int key_len, val_len, sec_len;
    long lval, e;

    cd = (cTemplateDict *)_return_resource (getThis(), le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "d_SetValueAndShowSection: error getting object");
        RETURN_FALSE;
    }

    if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "sssl", &sec, &sec_len, &key, &key_len, &val, &val_len, &e) == SUCCESS)
    {
        switch (e)
        {
            case 1:
                if (cd->root == true)
                    cd->d.SetEscapedValueAndShowSection (key, val, TemplateDictionary::xml_escape, sec);
                else
                    cd->p->SetEscapedValueAndShowSection (key, val, TemplateDictionary::xml_escape, sec);
                break;
            case 2:
                if (cd->root == true)
                    cd->d.SetEscapedValueAndShowSection (key, val, TemplateDictionary::javascript_escape, sec);
                else
                    cd->p->SetEscapedValueAndShowSection (key, val, TemplateDictionary::javascript_escape, sec);
                break;
            case 0:
            default:
                if (cd->root == true)
                    cd->d.SetEscapedValueAndShowSection (key, val, TemplateDictionary::html_escape, sec);
                else
                    cd->p->SetEscapedValueAndShowSection (key, val, TemplateDictionary::html_escape, sec);
        }
        RETURN_TRUE;
    }
    else if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "ssl", &sec, &sec_len, &key, &key_len, &lval) == SUCCESS)
    {
        if (cd->root == true)
        {
            cd->d.SetIntValue (key, lval);
            cd->d.ShowSection (sec);
        }
        else
        {
            cd->p->SetIntValue (key, lval);
            cd->p->ShowSection (sec);
        }
        RETURN_TRUE;
    }
    else if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "sss", &sec, &sec_len, &key, &key_len, &val, &val_len) == SUCCESS)
    {
        if (cd->root == true)
            cd->d.SetValueAndShowSection (key, val, sec);
        else
            cd->p->SetValueAndShowSection (key, val, sec);
        RETURN_TRUE;
    }
    else
        RETURN_FALSE;
}
/* }}} */

/* {{{ d_SetEscapedValueAndShowSection */
PHP_FUNCTION(d_SetEscapedValueAndShowSection)
{
    cTemplateDict *cd = NULL;
    const char *key = NULL, *val = NULL, sec = NULL;
    int key_len, val_len, sec_len;
    long e;

    cd = (cTemplateDict *)_return_resource (getThis(), le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "d_SetEscapedValueAndShowSection: error getting object");
        RETURN_FALSE;
    }

    if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "sssl", &sec, &sec_len, &key, &key_len, &val, &val_len, &e) == FAILURE)
        RETURN_FALSE;

    switch (e)
    {
        case 1:
            if (cd->root == true)
                cd->d.SetEscapedValueAndShowSection (key, val, TemplateDictionary::xml_escape, sec);
            else
                cd->p->SetEscapedValueAndShowSection (key, val, TemplateDictionary::xml_escape, sec);
            break;
        case 2:
            if (cd->root == true)
                cd->d.SetEscapedValueAndShowSection (key, val, TemplateDictionary::javascript_escape, sec);
            else
                cd->p->SetEscapedValueAndShowSection (key, val, TemplateDictionary::javascript_escape, sec);
            break;
        case 0:
        default:
            if (cd->root == true)
                cd->d.SetEscapedValueAndShowSection (key, val, TemplateDictionary::html_escape, sec);
            else
                cd->p->SetEscapedValueAndShowSection (key, val, TemplateDictionary::html_escape, sec);
    }
    RETURN_TRUE;
}
/* }}} */

PHP_FUNCTION(d_SetFormattedValue)
{
}

PHP_FUNCTION(d_SetEscapedFormattedValue)
{
}

/* {{{ d_SetGlobalValue */
PHP_FUNCTION(d_SetGlobalValue)
{
    cTemplateDict *cd = NULL;
    const char *key = NULL, *val;
    int key_len, val_len;

    if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "ss", &key, &key_len, &val, &val_len) == FAILURE)
        RETURN_FALSE;

    cd = (cTemplateDict *)_return_resource (getThis(), le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "d_SetGlobalValue: error getting object");
        RETURN_FALSE;
    }

    if (cd->root == true)
    cd->d.SetGlobalValue (key, val);
    cd->p->SetGlobalValue (key, val);
    RETURN_TRUE;
}
/* }}} */

/* {{{ d_AddSectionDictionary */
PHP_FUNCTION(d_AddSectionDictionary)
{
    zval *rv;
    cTemplateDict *cd, *ce;
    const char *sec;
    int sec_len;

    if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "s", &sec, &sec_len) == FAILURE)
        RETURN_FALSE;

    cd = (cTemplateDict *)_return_resource (getThis(), le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "d_AddSectionDictionary: error getting object");
        RETURN_FALSE;
    }

    ce = new cTemplateDict;
    ce->p = cd->d.AddSectionDictionary(sec);
    ce->root = false;

    if (ce != NULL)
    {
        MAKE_STD_ZVAL (rv);
        _add_resource (rv, ce, le_cTemplateDict TSRMLS_CC);
        SEPARATE_ZVAL (&rv);
        *return_value = *rv;
        FREE_ZVAL (rv);
        return;
    }
    RETURN_FALSE;
}
/* }}} */

/* {{{ d_ShowSection */
PHP_FUNCTION(d_ShowSection)
{
    cTemplateDict *cd = NULL;
    const char *sec = NULL;
    int sec_len;

    if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "s", &sec, &sec_len) == FAILURE)
        RETURN_FALSE;

    cd = (cTemplateDict *)_return_resource (getThis(), le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "d_ShowSection: error getting object");
        RETURN_FALSE;
    }

    if (cd->root == true)
        cd->d.ShowSection (sec);
    else
        cd->p->ShowSection (sec);
    RETURN_TRUE;
}
/* }}} */

/* {{{ d_AddIncludeDictionary */
PHP_FUNCTION(d_AddIncludeDictionary)
{
    zval *rv;
    cTemplateDict *cd, *ce;
    const char *val;
    int val_len;

    if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "s", &val, &val_len) == FAILURE)
        RETURN_FALSE;

    cd = (cTemplateDict *)_return_resource (getThis(), le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "d_AddIncludeDictionary: error getting object");
        RETURN_FALSE;
    }

    ce = new cTemplateDict;
    ce->p = cd->d.AddIncludeDictionary (val);
    ce->root = false;

    if (ce != NULL)
    {
        MAKE_STD_ZVAL (rv);
        _add_resource (rv, ce, le_cTemplateDict TSRMLS_CC);
        SEPARATE_ZVAL (&rv);
        *return_value = *rv;
        FREE_ZVAL (rv);
        return;
    }
    RETURN_FALSE;
}
/* }}} */

/* {{{ d_SetFilename */
PHP_FUNCTION(d_SetFilename)
{
    cTemplateDict *cd = NULL;
    const char *fn = NULL;
    int fn_len;

    if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "s", &fn, &fn_len) == FAILURE)
        RETURN_FALSE;

    cd = (cTemplateDict *)_return_resource (getThis(), le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "d_SetFilename: error getting object");
        RETURN_FALSE;
    }


    if (cd->root == true)
        cd->d.SetFilename (fn);
    else
        cd->p->SetFilename (fn);
    RETURN_TRUE;
}
/* }}} */

/* {{{ d_Dump */
PHP_FUNCTION(d_Dump)
{
    cTemplateDict * cd = NULL;

    if (ZEND_NUM_ARGS() != 0)
    {
        WRONG_PARAM_COUNT;
    }

    cd = (cTemplateDict *)_return_resource (getThis(), le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "d_Dump: error getting object");
        RETURN_FALSE;
    }

    if (cd->root == true)
        cd->d.Dump ();
    else
        cd->p->Dump ();
    RETURN_TRUE;
}
/* }}} */

/* {{{ d_DumpToString */
PHP_FUNCTION(d_DumpToString)
{
    cTemplateDict * cd = NULL;
    string ret;

    if (ZEND_NUM_ARGS() != 0)
    {
        WRONG_PARAM_COUNT;
    }

    cd = (cTemplateDict *)_return_resource (getThis(), le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "d_DumpToString: error getting object");
        RETURN_FALSE;
    }

    if (cd->root == true)
        cd->d.DumpToString (&ret);
    else
        cd->p->DumpToString (&ret);
    RETURN_STRINGL ((char *)ret.c_str(), ret.length(), 1);
}
/* }}} */

/* {{{ d_SetAnnotateOutput */
PHP_FUNCTION(d_SetAnnotateOutput)
{
    cTemplateDict * cd = NULL;
    const char *path = NULL;
    int path_len;

    if (zend_parse_parameters( ZEND_NUM_ARGS() TSRMLS_CC, "s", &path, &path_len) == FAILURE)
        RETURN_FALSE;

    cd = (cTemplateDict *)_return_resource (getThis(), le_cTemplateDict TSRMLS_CC);
    if (cd == NULL)
    {
        php_error (E_WARNING, "d_SetAnnotateOutput: error getting object");
        RETURN_FALSE;
    }

    if (cd->root == true)
        cd->d.SetAnnotateOutput (path);
    else
        cd->p->SetAnnotateOutput (path);
    RETURN_TRUE;
}
/* }}} */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
